def bresenham_line(x0, y0, x1, y1):
    points = []
    dx = abs(x1 - x0)
    dy = abs(y1 - y0)
    pk = 2 * dy - dx
    xk = x0
    yk = y0
    points.append((xk,yk))
    for _ in range(xk + 1):
        if pk < 0:
            xk = xk + 1
            pk = pk + 2 * dy
        elif pk >= 0:
            xk = xk + 1
            yk = yk + 1
            pk = pk + 2 * (dy - dx)
        points.append((xk,yk))
    return points


def translate(points, tx, ty):
    translated = [(x + tx, y + ty) for x, y in points]
    return translated

def rotate(points, angle):
    from math import radians, sin, cos
    rad = radians(angle)
    cos_val, sin_val = cos(rad), sin(rad)
    rotated = []
    for x, y in points:
        rotated.append((
            x * cos_val - y * sin_val,
            x * sin_val + y * cos_val
        ))
    return rotated

def scale(points, sx, sy):
    scaled = []
    for x, y in points:
        scaled.append((x * sx, y * sy))
    return scaled


from turtle import *

t = Turtle()

pts = bresenham_line(100,200,200,300)

t.penup()
print(pts)
for x,y in pts:
    t.goto(x,y)
    t.dot(3,'black')


new = rotate(pts, 90)

t.penup()
for x,y in new:
    t.goto(x,y)
    t.dot(3,'black')

done()


from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
import math

def rectangle(x1,y1,x2,y2,x3,y3,x4,y4,r,g,b):
    glColor3f(r,g,b)
    glBegin(GL_QUADS)
    glVertex2f(x1,y1); glVertex2f(x2,y2)
    glVertex2f(x3,y3); glVertex2f(x4,y4)
    glEnd()

def circle(radius,r,g,b):
    glColor3f(r,g,b)
    glBegin(GL_POLYGON)
    for i in range(100):
        a = 2*math.pi*i/100
        glVertex2f(radius*math.cos(a), radius*math.sin(a))
    glEnd()

def triangle(x1,y1,x2,y2,x3,y3,r,g,b):
    glColor3f(r,g,b)
    glBegin(GL_TRIANGLES)
    glVertex2f(x1,y1); glVertex2f(x2,y2); glVertex2f(x3,y3)
    glEnd()

def line(x1,y1,x2,y2,r,g,b):
    glColor3f(r,g,b)
    glBegin(GL_LINES)
    glVertex2f(x1,y1); glVertex2f(x2,y2)
    glEnd()

tree_dx, tree_dy = 0, 0   
STEP = 20            

def draw_exam():
    rectangle(0,0,0,400,800,400,800,0, 0.49,0.99,0.0)

    glPushMatrix()
    glTranslatef(700,700,0)
    circle(50, 1.0,0.87,0.0)
    glPopMatrix()

    glPushMatrix()
    glTranslatef(tree_dx, tree_dy, 0)
    rectangle(100,150,100,450,150,450,150,150, 0.31,0.20,0.08)
    glPushMatrix()
    glTranslatef(125,500,0)
    circle(100, 0.33,0.42,0.18)
    glPopMatrix()
    glPopMatrix()

    rectangle(250,100,250,450,650,450,650,100, 0.96,0.96,0.92)
    triangle(250,450,650,450,450,700, 0.65,0.16,0.16)
 
    rectangle(300,400,400,400,400,300,300,300, 0.24,0.24,0.24)
    rectangle(600,400,500,400,500,300,600,300, 0.24,0.24,0.24)
    line(600,350,500,350,1,1,1); line(550,400,550,300,1,1,1)
    line(300,350,400,350,1,1,1); line(350,400,350,300,1,1,1)

    rectangle(400,100,500,100,500,250,400,250, 0.40,0.26,0.13)

    glPushMatrix()
    glTranslatef(490,175,0)
    circle(10, 0,0.6,0)
    glPopMatrix()

def display():
    glClear(GL_COLOR_BUFFER_BIT)
    draw_exam()
    glFlush()

def keyboard(key, x, y):
    global tree_dx, tree_dy
    step = STEP
    if key == b'w':   
        tree_dy += step
    elif key == b's':   
        tree_dy -= step
    elif key == b'a':    
        tree_dx -= step
    elif key == b'd':   
        tree_dx += step
    glutPostRedisplay()

def special(key, x, y):   
    global tree_dx, tree_dy
    step = STEP
    if key == GLUT_KEY_UP:
        tree_dy += step
    elif key == GLUT_KEY_DOWN:
        tree_dy -= step
    elif key == GLUT_KEY_LEFT:
        tree_dx -= step
    elif key == GLUT_KEY_RIGHT:
        tree_dx += step
    glutPostRedisplay()

def init():
    glClearColor(0.53,0.81,0.92,1)
    gluOrtho2D(0, 800, 0, 800)

glutInit()
glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB)
glutInitWindowSize(800, 800)
glutCreateWindow(b"Move The Tree")
init()

glutDisplayFunc(display)
glutKeyboardFunc(keyboard)  
glutSpecialFunc(special)  

glutMainLoop()


from setuptools import setup, find_packages

setup(
    name="opengl_utils",
    version="0.1.6",

    packages=find_packages(),
    install_requires=[

    ],
    author="John Machanzy",
    author_email="jphn@machanzy.com",
    description="A utility package for OpenGL",
    long_description="a helper utility to help creating good gl graphics",
    long_description_content_type="text/markdown",
    url="https://github.com/johndoe/opengl_utils",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
)


from turtle import *

def main():

    t = Turtle()
    t.shape('turtle')
    t.fillcolor('green')
    t.color('blue')
    t.pensize(5)
    t.speed(2)
    t.circle(100,steps=8)
    t.penup()
    t.left(90)
    t.forward(100)
    t.pendown()
    for _ in range(7):
        t.left(130)
        t.forward(30)
        t.left(50)
        t.forward(30)
        t.left(130)
        t.forward(30)
        t.left(50)
        t.forward(30)
        t.left(51)
    t.right(120)
    t.penup()
    t.forward(100)
    t.pendown()
    for _ in range(7):
        t.left(130)
        t.forward(30)
        t.left(50)
        t.forward(30)
        t.left(130)
        t.forward(30)
        t.left(50)
        t.forward(30)
        t.left(51)


    t.color('black')
    t.pensize(10)
    t.left(50)
    t.penup()
    t.forward(55)
    t.left(80)
    t.pd()

    t.forward(100)
    t.right(90)
    t.forward(100)
    t.backward(100)
    t.right(90)
    t.forward(100)
    t.left(90)
    t.forward(100)
    t.backward(100)
    t.right(90)
    t.forward(100)
    t.left(90)
    t.forward(100)
    done()




main()


from turtle import *


def bresenham_circle(r):
    pk = 3 - 2 * r
    xk = 0
    yk = r
    points = [(xk,yk)]
    while (xk < yk):
        if (pk<0):
            xk = xk + 1
            pk = pk + 4 * xk + 6
        elif (pk>=0):
            xk = xk + 1
            yk = yk - 1
            pk = pk + 4 * (xk - yk) + 10
        if (xk != yk):
            points.append((xk,yk))
    return points

def midpoint_circle(r):
    pk = 1 - r
    xk = 0
    yk = r
    points = [(xk, yk)]
    while (xk < yk):
        if pk >= 0:
            xk = xk + 1
            yk = yk - 1 
            pk = pk + 2 * (xk - yk) + 1
        elif pk < 0:
            xk = xk + 1
            yk = yk 
            pk = pk + 2 * (xk) + 1
        if (xk != yk):
            points.append((xk, yk))
    return points


x0 , y0 = 100, 100
octant1 = bresenham_circle(90)
octant2 = [(y, x) for x, y in octant1]
octant3 = [(-x, y) for x, y in octant1]
octant4 = [(-y, x) for x, y in octant1]
octant5 = [(-x, -y) for x, y in octant1]
octant6 = [(-y, -x) for x, y in octant1]
octant7 = [(x, -y) for x, y in octant1]
octant8 = [(y, -x) for x, y in octant1]

circle_octants = [octant1, octant2, octant3, octant4, octant5, octant6, octant7, octant8]
circle_quadrants = [
    octant1 + octant2,
    octant3 + octant4,
    octant5 + octant6,
    octant7 + octant8
]

print("Circle Octants")
for i in range(len(circle_octants)):
    print(f"Octant {i+1}")
    print(circle_octants[i])

print(50 * "#")

print("Circle Octants")
for i in range(len(circle_quadrants)):
    print(f"Quadrant {i+1}")
    print(circle_quadrants[i])

t = Turtle()
t.speed(0)
t.penup()

for octant in circle_quadrants:
    for x, y in octant:
        t.goto(x + x0, y + y0)
        t.dot(3, "black")

t. goto(x0-70, y0)
t.pendown()
t.pensize(5)
t.write("Tawfiq Bhyry", font=("Rubik", 12, "bold"))

t.hideturtle()
done()

def bresenham_line(x0, y0, x1, y1):
    points = []
    dx = abs(x1 - x0)
    dy = abs(y1 - y0)
    pk = 2 * dy - dx
    xk = x0
    yk = y0
    points.append((xk,yk))
    for _ in range(xk + 1):
        if pk < 0:
            xk = xk + 1
            pk = pk + 2 * dy
        elif pk >= 0:
            xk = xk + 1
            yk = yk + 1
            pk = pk + 2 * (dy - dx)
        points.append((xk,yk))
    return points


def midpoint_line(x0, y0, x1, y1):
    points = []
    dx = abs(x1 - x0)
    dy = abs(y1 - y0)
    d = 2 * dy - dx
    delta_d = 2 * (dy - dx)
    xk = x0
    yk = y0
    points.append((xk,yk))
    for _ in range(xk + 1):
        if d < 0:
            xk = xk + 1
            d = d + (2 * dx)
        elif d >= 0:
            xk = xk + 1
            yk = yk + 1
            d = delta_d
        points.append((xk,yk))
    return points


